import AddIcon from "@mui/icons-material/Add";
import RefreshIcon from "@mui/icons-material/Refresh";
import { IconButton, Tooltip } from "@mui/material";
import { DataGrid, GridColDef } from "@mui/x-data-grid";
import { Account, AccountApi } from "../api/AccountApi";
import { useAccounts } from "../hooks/AccountsListProvider";
import { useAlert } from "../hooks/context_providers/AlertDialogProvider";
import { useSnackbar } from "../hooks/context_providers/SnackbarProvider";
import { MoneyMgrWebRouteContainer } from "../widgets/MoneyMgrWebRouteContainer";
import { TimeWidget } from "../widgets/TimeWidget";

export function AccountsRoute(): React.ReactElement {
  const alert = useAlert();
  const snackbar = useSnackbar();

  const accounts = useAccounts();

  const list = accounts.list.list;

  const setDefaultAccount = async (account: Account) => {
    try {
      await AccountApi.SetDefaultAccount(account);

      snackbar("Default account successfully updated!");

      await accounts.reload();

      return accounts.get(account.id);
    } catch (e) {
      console.error("Failed to change default account!", e);
      alert(`Failed to change default account! ${e}`);
      return account;
    }
  };

  const updateAccount = async (account: Account) => {
    try {
      await AccountApi.Update(account);

      snackbar("Account successfully updated!");

      await accounts.reload();

      return accounts.get(account.id);
    } catch (e) {
      console.error("Failed to update account!", e);
      alert(`Failed to update account! ${e}`);
      return account;
    }
  };

  const columns: GridColDef<(typeof list)[number]>[] = [
    { field: "id", headerName: "ID", flex: 1 },
    { field: "name", headerName: "Name", flex: 6, editable: true },
    {
      field: "time_create",
      headerName: "Time created",
      flex: 2,
      renderCell(params) {
        return <TimeWidget time={params.row.time_create} />;
      },
    },
    {
      field: "time_update",
      headerName: "Time updated",
      flex: 2,
      renderCell(params) {
        return <TimeWidget time={params.row.time_update} />;
      },
    },
    {
      field: "default_account",
      headerName: "Default",
      flex: 1,
      type: "boolean",
      editable: true,
    },
  ];

  return (
    <MoneyMgrWebRouteContainer
      label="Accounts"
      actions={
        <span>
          <Tooltip title="Create a new account">
            <IconButton onClick={undefined}>
              <AddIcon />
            </IconButton>
          </Tooltip>
          <Tooltip title="Refresh accounts list">
            <IconButton onClick={undefined}>
              <RefreshIcon />
            </IconButton>
          </Tooltip>
        </span>
      }
    >
      <DataGrid<Account>
        rows={list}
        columns={columns}
        disableRowSelectionOnClick
        processRowUpdate={(updated, original) => {
          if (updated.default_account && !original.default_account) {
            return setDefaultAccount(updated);
          }

          if (updated.name !== original.name) {
            return updateAccount(updated);
          }

          return original as any;
        }}
      />
    </MoneyMgrWebRouteContainer>
  );
}
